
const subTableId = "#subTable";
const subModalId = "#subFormModal";


// region Init Module

function initSubscriptionsModule() {
    initializeDataTable(
        subTableId,
        [
            {
                title: "Name",
                data: "name",
                className: "text-truncate col-3",
                render: renderEditColumn
            },
            {
                title: "URL",
                data: "url",
                className: "text-truncate col-4",
                render: function(data, type) {
                    const url = sanitise(data);
                    return `<a href="${url}" class="btn btn-link text-start text-decoration-none" target="_blank">${url}</a>`;
                }
            },
            {
                title: "Channels",
                data: "channels",
            },
            {
                title: "Rules",
                data: "unique_rules",
                className: "text-center text-nowrap col-1",
                render: function(data, type) {
                    let badges = $("<div>");
                    data.forEach(function(rule, idx) {
                        let badge = $(`<span class="badge text-bg-secondary">${rule}</span>`)  // TODO: translate to rule.name
                        if (idx > 0) { badge.addClass("ms-2") }
                        badges.append(badge);
                    });

                    return badges.html();
                }
            },
            {
                title: "Created At",
                data: "created_at",
                className: "col-3",
                render: function(data, type) {
                    let dateTime = new Date(data);
                    return $(`
                        <span data-bs-trigger="hover focus"
                            data-bs-html="true"
                            data-bs-custom-class="text-center"
                            data-bs-toggle="popover"
                            data-bs-content="${formatStringDate(dateTime, "%a, %D %B, %Y<br>%H:%M:%S")}">
                            ${formatStringDate(dateTime, "%D, %b %Y")}
                        </span>
                    `).popover()[0];
                }
            },
            { 
                title: "Enabled",
                data: "active",
                orderable: false,
                className: "text-center form-switch col-1",
                render: function(data, type) {
                    return `<input type="checkbox" class="sub-toggle-active form-check-input ms-0" ${data ? "checked" : ""} />`
                }
            }
        ]
    );
}


// region Load Data

$(document).on("selectedServerChange", async function() {
    await loadSubscriptionData();
});

$(subTableId).on("doDataLoad", async function() {
    await loadSubscriptionData();
})

async function loadSubscriptionData() {
    if (!selectedServer) {
        return;
    }

    setTableFilter(subTableId, "server", selectedServer.id);
    await loadTableData(subTableId, `/api/subscriptions/`, "GET");
}


// region Table Switches

$(subTableId).on("change", ".sub-toggle-active", async function() {

    // Temporarily disable all switches to prevent spam.
    $(subTableId).find(".sub-toggle-active").prop("disabled", true);
    setTimeout(() => { $(subTableId).find(".sub-toggle-active").prop("disabled", false); }, 800);

    let active = $(this).prop("checked");
    let id = $(subTableId).DataTable().row($(this).closest("tr")).data().id;
    let sub = await ajaxRequest(`/api/subscriptions/${id}/`, "GET");
    sub.active = active;
    let formData = objectToFormData(sub);
    await ajaxRequest(`/api/subscriptions/${id}/`, "PUT", formData);
});


// region Table Delete Buttons

$(subModalId).find(".modal-del-btn").on("click", async function() {
    $(subModalId).modal("hide");

    const id = parseInt($(subModalId).data("primary-key"));
    const subscription = $(subTableId).DataTable().row((idx, row) => { return row.id === id }).data();
    const name = sanitise(subscription.name);

    await confirmationModal(
        `Delete a Subscription`,
        `Do you wish to permanently delete <b>${name}</b>?`,
        "danger",
        async () => {
            await ajaxRequest(`/api/subscriptions/${subscription.id}/`, "DELETE");
            setTimeout(() => { $(subTableId).trigger("doDataLoad") }, 600);
        },
        () => { $(subModalId).modal("show") }
    );
})

getTableFiltersComponent(subTableId).find(".table-del-btn").on("click", async function() {
    const rows = getSelectedTableRows(subTableId);

    const names = rows.map(row => row.name);
    const namesString = arrayToHtmlList(names, true).prop("outerHTML");
    const isMany = names.length > 1;
    
    await confirmationModal(
        `Delete ${isMany ? "Many Subscriptions" : "a Subscription"}`,
        `Do you wish to permanently delete ${isMany ? "these" : "this"} <b>${names.length}</b> subscription${isMany ? "s" : ""}?<br><br>${namesString}`,
        "danger",
        async () => {
            rows.forEach(async row => {
                await ajaxRequest(`/api/subscriptions/${row.id}/`, "DELETE");
            });
    
            setTimeout(() => { $(subTableId).trigger("doDataLoad") }, 600);
        },
        null
    )
});


// region New/Edit Modal

$(subTableId).closest('.js-tableBody').siblings('.js-tableFilters').on("click", ".table-new-btn", async function() {
    await openSubModal(-1);
});

$(subTableId).on("click", ".edit-modal", async function() {
    const id = $(subTableId).DataTable().row($(this).closest("tr")).data().id;
    await openSubModal(id);
})

async function openSubModal(id) {
    await loadChannelOptions();
    await openDataModal(subModalId, id, id !== -1 ? `/api/subscriptions/${id}/`: null);
}

$(subModalId).on("submit", async function(event) {
    event.preventDefault();
    await onModalSubmit(
        $(subModalId),
        $(subTableId),
        "/api/subscriptions/"
    );
});


// region Load Modal Options

$(document).on("selectedServerChange", async function() {
    await loadSubModalOptions(
        $(subModalId).find('[data-field="message_style"]'),
        `/api/message-styles/?server=${selectedServer.id}`
    );
    await loadSubModalOptions(
        $(subModalId).find('[data-field="filters"]'),
        `/api/filters/?server=${selectedServer.id}`
    );
    
});

$(document).ready(async function() {
    await loadSubModalOptions(
        $(subModalId).find('[data-field="unique_rules"]'),
        "/api/unique-content-rules/"
    );
})

async function loadSubModalOptions($input, url) {
    // Disable and clear input
    $input.val("").change();
    $input.prop("disabled", true);

    // Delete existing options
    $input.find("option").each(function() {
        if ($(this).val()) {
            $(this).remove();
        }
    });

    // Load new values
    const data = await ajaxRequest(url, "GET");
    data.results.forEach(item => {
        $input.append($(
            "<option>",
            {text: item.name, value: item.id}
        ));
    });

    // Re-enable input
    $input.prop("disabled", false);
}

// Channel options aren't loaded from an API, like other options.
function loadChannelOptions() {
    $input = $(subModalId).find('[data-field="channels"]');

    $input.val("").change();
    $input.prop("disabled", true);

    $input.find("option").each(function() {
        if ($(this).val()) {
            $(this).remove();
        }
    });

    const data = loadedChannels(selectedServer.id);
    data.forEach(item => {
        $input.append($(
            "<option>",
            {text: `#${item.name}`, value: BigInt(item.id)}
        ));
    })

    $input.prop("disabled", false);
}